// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/util/CommandLine.hpp>
#include <anna/core/DataBlock.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/comm/comm.hpp>

#include <anna/app/Application.hpp>

class MyCodec : public Codec {
public:
   MyCodec () : Codec (0xaa, false) 
   {
      attach ("string", a_string);
      attach ("integer", a_integer);
   }

   void set (const char* value) throw () { a_string = value; }
   void set (const int value) throw () { a_integer = value; }

   const std::string& getString () const throw () { return a_string; }
   const int getInteger () const throw () { return a_integer; }

private:
   std::string a_string;
   int a_integer;
};

class Test : public anna::app::Application {
public:
   Test ();

private:
   MyCodec a_input;

   void show (const LargeBinaryCodec& lbc) throw (anna::RuntimeException);

   void run () throw (anna::RuntimeException);
};

using namespace std;
using namespace anna;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;
   TraceWriter traceWriter ("my.trace", 2048000);

   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::initialize ("testfunctions", &traceWriter);
      Logger::setLevel (Logger::Debug);

      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }

   return 0;
}

Test::Test () :
   anna::app::Application ("testfunctions", "Comprobacin de la clase Codec", "1.0")
{
   CommandLine& commandLine (CommandLine::instantiate ());

   commandLine.add ("i", CommandLine::Argument::Mandatory, "Enterno a enviar");
}

void Test::run ()
   throw (RuntimeException)
{
   CommandLine& cm (CommandLine::instantiate ());
   LargeBinaryCodec input (1, false);
   LargeBinaryCodec output (1, false);

   a_input.set ("12345");
   a_input.set (cm.getIntegerValue ("i"));
   input += a_input.code ();

   a_input.set ("6789");
   a_input.set (cm.getIntegerValue ("i") + 10);
   input += a_input.code ();

   a_input.set ("abcdefg");
   a_input.set (cm.getIntegerValue ("i") + 20);
   input += a_input.code ();

   cout << "------------- Input LargeBinaryCodec: " << endl;
   show (input);

   cout << "------------- Output LargeBinaryCodec: " << endl;
   const DataBlock& all = input.code ();  // Codifica el LBC.
   output.decode (all);                   // Lo decodific
   show (output);
}

void Test::show (const LargeBinaryCodec& lbc) 
   throw (RuntimeException)
{
   MyCodec output;

   cout << "Size: " << lbc.size () << endl;

   for (LargeBinaryCodec::const_iterator ii = lbc.begin (), maxii = lbc.end (); ii != maxii; ii ++) {
      const DataBlock& chunk = *LargeBinaryCodec::data (ii);

      Logger::debug (anna::functions::asString (chunk), ANNA_FILE_LOCATION);

      output.decode (chunk);

      cout << "String: " << output.getString () << endl;
      cout << "Integer: " << output.getInteger () << endl << endl;
   }

   cout << endl;
}

