// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_helpers_base_defines_hpp
#define anna_diameter_helpers_base_defines_hpp


// Local
#include <anna/diameter/helpers/defines.hpp>

// STL
#include <string>


/** Contextualized definitions (avp id) */
#define anna_diameter_helpers_define_avp(avpname,code)  anna_diameter_helpers_define_avp_in_context(avpname,code,base)

/** Contextualized definitions (avp id and values) */
#define anna_diameter_helpers_define_avp_with_values(avpname,code)  anna_diameter_helpers_define_avp_with_values_in_context(avpname,code,base)


namespace anna {

namespace diameter {

namespace helpers {

namespace base {


/**
* Avps
*/
anna_diameter_helpers_define_avp(Host_IP_Address, 257);
anna_diameter_helpers_define_avp(Auth_Application_Id, 258);
anna_diameter_helpers_define_avp_with_values(Acct_Application_Id, 259) {
  enum v_ {
    ERICSSON_SCAP = 19302
  };
};

anna_diameter_helpers_define_avp(Vendor_Specific_Application_Id, 260);
anna_diameter_helpers_define_avp_with_values(Redirect_Host_Usage, 261) {
  enum v_ {
    DONT_CACHE = 0,
    ALL_SESSION = 1,
    ALL_REALM = 2,
    REALM_AND_APPLICATION = 3,
    ALL_APPLICATION = 4,
    ALL_HOST = 5,
    ALL_USER = 6
  };
};

anna_diameter_helpers_define_avp(Redirect_Max_Cache_Time, 262);
anna_diameter_helpers_define_avp(Session_Id, 263);
anna_diameter_helpers_define_avp(Origin_Host, 264);
anna_diameter_helpers_define_avp(Supported_Vendor_Id, 265);
anna_diameter_helpers_define_avp(Vendor_Id, 266);
anna_diameter_helpers_define_avp(Firmware_Revision, 267);
anna_diameter_helpers_define_avp_with_values(Result_Code, 268) {
  enum v_ {
    // (Informational)
    // BaseProtocol
    DIAMETER_MULTI_ROUND_AUTH = 1001,
    // (Success)
    // BaseProtocol
    DIAMETER_SUCCESS = 2001,
    DIAMETER_LIMITED_SUCCESS = 2002,
    // (Protocol Errors)
    // BaseProtocol
    DIAMETER_COMMAND_UNSUPPORTED = 3001,
    DIAMETER_UNABLE_TO_DELIVER = 3002,
    DIAMETER_REALM_NOT_SERVED = 3003,
    DIAMETER_TOO_BUSY = 3004,
    DIAMETER_LOOP_DETECTED = 3005,
    DIAMETER_REDIRECT_INDICATION = 3006,
    DIAMETER_APPLICATION_UNSUPPORTED = 3007,
    DIAMETER_INVALID_HDR_BITS = 3008,
    DIAMETER_INVALID_AVP_BITS = 3009,
    DIAMETER_UNKNOWN_PEER = 3010,
    // (Transient Failures)
    // BaseProtocol
    DIAMETER_AUTHENTICATION_REJECTED = 4001,
    DIAMETER_OUT_OF_SPACE = 4002,
    ELECTION_LOST = 4003,
    // MobileIPv4Application
    DIAMETER_ERROR_MIP_REPLY_FAILURE = 4005,
    DIAMETER_ERROR_HA_NOT_AVAILABLE = 4006,
    DIAMETER_ERROR_BAD_KEY = 4007,
    DIAMETER_ERROR_MIP_FILTER_NOT_SUPPORTED = 4008,
    // DCCA
    DIAMETER_END_USER_SERVICE_DENIED = 4010,
    DIAMETER_CREDIT_CONTROL_NOT_APPLICABLE = 4011,
    DIAMETER_CREDIT_LIMIT_REACHED = 4012,
    // Ericsson SCAP
    SCAP__DIAMETER_END_USER_SERVICE_DENIED = 4241,
    // (Permanent Failure)
    // BaseProtocol
    DIAMETER_AVP_UNSUPPORTED = 5001,
    DIAMETER_UNKNOWN_SESSION_ID = 5002,
    DIAMETER_AUTHORIZATION_REJECTED = 5003,
    DIAMETER_INVALID_AVP_VALUE = 5004,
    DIAMETER_MISSING_AVP = 5005,
    DIAMETER_RESOURCES_EXCEEDED = 5006,
    DIAMETER_CONTRADICTING_AVPS = 5007,
    DIAMETER_AVP_NOT_ALLOWED = 5008,
    DIAMETER_AVP_OCCURS_TOO_MANY_TIMES = 5009,
    DIAMETER_NO_COMMON_APPLICATION = 5010,
    DIAMETER_UNSUPPORTED_VERSION = 5011,
    DIAMETER_UNABLE_TO_COMPLY = 5012,
    DIAMETER_INVALID_BIT_IN_HEADER = 5013,
    DIAMETER_INVALID_AVP_LENGTH = 5014,
    DIAMETER_INVALID_MESSAGE_LENGTH = 5015,
    DIAMETER_INVALID_AVP_BIT_COMBO = 5016,
    DIAMETER_NO_COMMON_SECURITY = 5017,
    // MobileIPv4Application
    DIAMETER_ERROR_NO_FOREIGN_HA_SERVICE = 5024,
    DIAMETER_ERROR_END_TO_END_MIP_KEY_ENCRYPTION = 5025,
    // DCCA
    DIAMETER_USER_UNKNOWN = 5030,
    DIAMETER_RATING_FAILED = 5031,
    // Rx
    REQUESTED_SERVICE_NOT_AUTHORIZED = 5063,
    IP_CAN_SESSION_NOT_AVAILABLE = 5065,
    // Ericsson SCAP
    DIAMETER_END_USER_NOT_FOUND = 5241
  };
};

anna_diameter_helpers_define_avp(Product_Name, 269);
anna_diameter_helpers_define_avp_with_values(Session_Binding, 270) {
  enum v_ {
    RE_AUTH = 1,
    STR = 2,
    ACCOUNTING = 4
  };
};

anna_diameter_helpers_define_avp_with_values(Session_Server_Failover, 271) {
  enum v_ {
    REFUSE_SERVICE = 0,
    TRY_AGAIN = 1,
    ALLOW_SERVICE = 2,
    TRY_AGAIN_ALLOW_SERVICE = 3
  };
};

anna_diameter_helpers_define_avp(Multi_Round_Time_Out, 272);
anna_diameter_helpers_define_avp_with_values(Disconnect_Cause, 273) {
  enum v_ {
    REBOOTING = 0,
    BUSY = 1,
    DO_NOT_WANT_TO_TALK_TO_YOU = 2
  };
};

anna_diameter_helpers_define_avp_with_values(Auth_Request_Type, 274) {
  enum v_ {
    RESERVED = 0,
    AUTHENTICATE_ONLY = 1,
    AUTHORIZE_ONLY = 2,
    AUTHORIZE_AUTHENTICATE = 3
  };
};

anna_diameter_helpers_define_avp(Auth_Grace_Period, 276);
anna_diameter_helpers_define_avp_with_values(Auth_Session_State, 277) {
  enum v_ {
    STATE_MAINTAINED = 0,
    NO_STATE_MAINTAINED = 1
  };
};

anna_diameter_helpers_define_avp(Origin_State_Id, 278);
anna_diameter_helpers_define_avp(Failed_AVP, 279);
anna_diameter_helpers_define_avp(Proxy_Host, 280);
anna_diameter_helpers_define_avp(Error_Message, 281);
anna_diameter_helpers_define_avp(Route_Record, 282);
anna_diameter_helpers_define_avp(Destination_Realm, 283);
anna_diameter_helpers_define_avp(Proxy_Info, 284);
anna_diameter_helpers_define_avp_with_values(Re_Auth_Request_Type, 285) {
  enum v_ {
    AUTHORIZE_ONLY = 0,
    AUTHORIZE_AUTHENTICATE = 1
  };
};

anna_diameter_helpers_define_avp(Accounting_Sub_Session_Id, 287);
anna_diameter_helpers_define_avp(Authorization_Lifetime, 291);
anna_diameter_helpers_define_avp(Redirect_Host, 292);
anna_diameter_helpers_define_avp(Destination_Host, 293);
anna_diameter_helpers_define_avp(Error_Reporting_Host, 294);
anna_diameter_helpers_define_avp_with_values(Termination_Cause, 295) {
  enum v_ {
    DIAMETER_LOGOUT = 1,
    DIAMETER_SERVICE_NOT_PROVIDED = 2,
    DIAMETER_BAD_ANSWER = 3,
    DIAMETER_ADMINISTRATIVE = 4,
    DIAMETER_LINK_BROKEN = 5,
    DIAMETER_AUTH_EXPIRED = 6,
    DIAMETER_USER_MOVED = 7,
    DIAMETER_SESSION_TIMEOUT = 8
  };
};

anna_diameter_helpers_define_avp(Origin_Realm, 296);
anna_diameter_helpers_define_avp(Experimental_Result, 297);
anna_diameter_helpers_define_avp(Experimental_Result_Code, 298);
anna_diameter_helpers_define_avp_with_values(Inband_Security_Id, 299) {
  enum v_ {
    NO_INBAND_SECURITY = 0,
    TLS = 1
  };
};

anna_diameter_helpers_define_avp(E2E_Sequence_AVP, 300); // deprecated
anna_diameter_helpers_define_avp_with_values(Accounting_Record_Type, 480) {
  enum v_ {
    EVENT_RECORD = 1,
    START_RECORD = 2,
    INTERIM_RECORD = 3,
    STOP_RECORD = 4
  };
};

anna_diameter_helpers_define_avp(Accounting_Interim_Interval, 482);
anna_diameter_helpers_define_avp_with_values(Accounting_Realtime_Required, 483) {
  enum v_ {
    DELIVER_AND_GRANT = 1,
    GRANT_AND_STORE = 2,
    GRANT_AND_LOSE = 3
  };
};

anna_diameter_helpers_define_avp(Accounting_Record_Number, 485);


/**
* Commands
*/
anna_diameter_helpers_define_command_request(Abort_Session_Request, 274);
anna_diameter_helpers_define_command_answer(Abort_Session_Answer, 274);
anna_diameter_helpers_define_command_request(Accounting_Request, 271);
anna_diameter_helpers_define_command_answer(Accounting_Answer, 271);
anna_diameter_helpers_define_command_request(Capabilities_Exchange_Request, 257);
anna_diameter_helpers_define_command_answer(Capabilities_Exchange_Answer, 257);
anna_diameter_helpers_define_command_request(Device_Watchdog_Request, 280);
anna_diameter_helpers_define_command_answer(Device_Watchdog_Answer, 280);
anna_diameter_helpers_define_command_request(Disconnect_Peer_Request, 282);
anna_diameter_helpers_define_command_answer(Disconnect_Peer_Answer, 282);
anna_diameter_helpers_define_command_request(Re_Auth_Request, 258);
anna_diameter_helpers_define_command_answer(Re_Auth_Answer, 258);
anna_diameter_helpers_define_command_request(Session_Termination_Request, 275);
anna_diameter_helpers_define_command_answer(Session_Termination_Answer, 275);


}
}
}
}


#undef anna_diameter_helpers_define_avp
#undef anna_diameter_helpers_define_avp_with_values

#endif
