// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_helpers_dcca_functions_hpp
#define anna_diameter_helpers_dcca_functions_hpp


#include <anna/core/RuntimeException.hpp>

#include <anna/diameter/helpers/dcca/defines.hpp>

// STL
#include <string>

//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------

namespace anna {
class DataBlock;
}



namespace anna {

namespace diameter {

namespace helpers {

namespace dcca {

/**
* Type of charging context
*/
struct ChargingContext { enum _v { Unknown = -1, Data, Voice, Content, SMS, MMS }; };


struct ChargingContextAndDomainSuffix {

  /** Charging context and domain suffic for data traffic */
  static const char Data[];

  /** Charging context and domain suffic for voice traffic */
  static const char Voice[];

  /** Charging context and domain suffic for content traffic */
  static const char Content[];

  /** Charging context and domain suffic for SMS traffic */
  static const char SMS[];

  /** Charging context and domain suffic for MMS traffic */
  static const char MMS[];
};


struct functions {

  // getters & helpers

  /**
  * Subscription-Id-Data for provided Subscription-Id-Type within a command.
  *
  * @param db Provided Message DataBlock
  * @param Subscription-Id-Type value to filter the search. END_USER_E164 by default.
  *
  * @return Subscription-Id-Data for type provided. Empty string when not found.
  */
  static std::string getSubscriptionIdData(const anna::DataBlock &db,
      int subscriptionIdType = AVPVALUES__Subscription_Id_Type::END_USER_E164) throw(anna::RuntimeException);


  /**
  * Service-Context-Id from datablock provided.
  *
  * @param db Provided Message DataBlock
  * @param chargingContext Detected charging context
  *
  * @return Service-Context-Id
  */
  static std::string getServiceContextId(const anna::DataBlock &db, ChargingContext::_v &chargingContext) throw(anna::RuntimeException);

  /**
  * Service-Context-Id from datablock provided.
  *
  * @param db Provided Message DataBlock
  *
  * @return Service-Context-Id
  */
  static std::string getServiceContextId(const anna::DataBlock &db) throw(anna::RuntimeException) {
    ChargingContext::_v dummy;
    return getServiceContextId(db, dummy);
  }
};


}
}
}
}


#endif

