// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <algorithm>

#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>

#include <anna/xml/Node.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/Server.hpp>
#include <anna/comm/ClientSocket.hpp>

using namespace std;
using namespace anna;

/*
 * Se invoca desde el handler::RemoteConnection::finalize y debe estar protegido por la SSCC del Communicator
 */
void comm::ConnectionRecover::annotateFault(comm::Server* server)
throw() {
  if(server->autoRecovery() == false)
    return;

  if(find(a_breaks.begin(), a_breaks.end(), server) != a_breaks.end())
    return;

  if(a_isRunning == false) {
    a_nextTime = functions::millisecond() + a_communicator.getRecoveryTime();
    a_isRunning = true;
  }

  a_breaks.push_back(server);
  a_recovering = a_breaks.begin();
  string msg("comm::ConnectionRecover::fault | ");
  msg += server->asString();
  msg += anna::functions::asText(" | NextTime: ", a_nextTime);
  Logger::error(msg, ANNA_FILE_LOCATION);
}

/*
 * Se invoca desde el Communicator y en MT debe estar protegido por la SSCC del Communicator
 */
void comm::ConnectionRecover::tryRecover()
throw() {
  LOGMETHOD(TraceMethod traceMethod(Logger::Local7, "comm::ConnectionRecover", "tryRecover", ANNA_FILE_LOCATION));
  Millisecond now(functions::millisecond());

  if(now < a_nextTime)
    return;

  if(a_recovering == a_breaks.end())
    return;

  Millisecond maxTime = now + a_communicator.getTryingConnectionTime();
  break_iterator beginning = a_recovering;
  Server* server;

  while(now < maxTime && a_breaks.size() > 0) {
    server = *a_recovering;

    try {
      server->connect();
      LOGNOTICE(
        Logger::write(Logger::Notice, "Recover", server->asString(), ANNA_FILE_LOCATION)
      );
      a_recovering = a_breaks.erase(a_recovering);
    } catch(Exception& ex) {
      a_recovering ++;
    }

    if(a_recovering == a_breaks.end())
      a_recovering = a_breaks.begin();

    if(a_recovering == beginning)
      break;

    now = functions::millisecond();
  }

  a_isRunning = (a_breaks.size() > 0);
  a_nextTime = now + a_communicator.getRecoveryTime();
  LOGDEBUG(
    string msg("comm::ConnectionRecover::recover | Running: ");
    msg += anna::functions::asString(a_isRunning);
    msg += anna::functions::asText(" | N: ", (int) a_breaks.size());
    msg += anna::functions::asText(" | NextTime: ", a_nextTime);
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

/*
bool comm::ConnectionRecover::contains (comm::Server* server) const
   throw ()
{
   Guard guard (a_mutex, "comm::ConnectionRecover::contains");
   return std::find (a_breaks.begin (), a_breaks.end (), server) != a_breaks.end ();
}

void comm::ConnectionRecover::erase (comm::Server* server)
   throw ()
{
   Guard guard (a_mutex, "comm::ConnectionRecover::erase");

   vector <Server*>::iterator ii = find (a_breaks.begin (), a_breaks.end (), server);

   if (ii == a_breaks.end ())
      return;

   if (ii == a_recovering)
      if ((a_recovering = a_breaks.erase (ii)) == a_breaks.end ())
         a_recovering = a_breaks.begin ();

   a_isRunning = (a_breaks.size () > 0);
}
*/

xml::Node* comm::ConnectionRecover::asXML(xml::Node* parent) const
throw(RuntimeException) {
  xml::Node* connectionRecover = parent->createChild("comm.ConnectionRecover");
  connectionRecover->createAttribute("Active", functions::asString(a_isRunning));

  for(std::vector <Server*>::const_iterator ii = a_breaks.begin(), maxii = a_breaks.end(); ii != maxii; ii ++)
    (*ii)->asXML(connectionRecover);

  return connectionRecover;
}



