// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter/codec/basetypes/Time.hpp>

#include <anna/time/Date.hpp> // TIMESTAMP_OFFSET_NTP1900_OVER_UNIX1970


//------------------------------------------------------------------------------
//---------------------------------------------------------- Time::updateBasic()
//------------------------------------------------------------------------------
void anna::diameter::codec::basetypes::Time::updateBasic() throw(anna::RuntimeException) {
  std::string result;
  result += (char)(a_ntpTimestamp >> 24);
  result += (char)(a_ntpTimestamp >> 16);
  result += (char)(a_ntpTimestamp >> 8);
  result += (char)a_ntpTimestamp;
  OctetString::setValue(result);
}


//------------------------------------------------------------------------------
//--------------------------------------------------- Time::setPrintableString()
//------------------------------------------------------------------------------
void anna::diameter::codec::basetypes::Time::setPrintableString(const char * printableString) throw(anna::RuntimeException) {
  setTimestamp(atoi(printableString));
}


//------------------------------------------------------------------------------
//------------------------------------------------------------- Time::asString()
//------------------------------------------------------------------------------
std::string anna::diameter::codec::basetypes::Time::asString() throw(anna::RuntimeException) {
  std::string result = asPrintableString();
  anna::time::Date date;
  date.storeNtp(a_ntpTimestamp);
  result += " <"; result += date.asString(); result += ">";
  return result;
}


//------------------------------------------------------------------------------
//--------------------------------------------------------------- Time::decode()
//------------------------------------------------------------------------------
void anna::diameter::codec::basetypes::Time::decode(const char* buffer, const int size) throw(anna::RuntimeException) {
  if(!buffer)
    throw anna::RuntimeException("Time::decode | Null Buffer provided", ANNA_FILE_LOCATION);

  if(size != 4)
    throw anna::RuntimeException("Time::decode | Buffer length must be 4 bytes", ANNA_FILE_LOCATION);

  a_ntpTimestamp = (((U32)buffer[0] << 24) & 0xFF000000) +
                   (((U32)buffer[1] << 16) & 0x00FF0000) +
                   (((U32)buffer[2] << 8)  & 0x0000FF00) +
                   ((U32)buffer[3]         & 0x000000FF);
  // Base class decode()
  OctetString::decode(buffer, size);
}


//------------------------------------------------------------------------------
//--------------------------------------------------------- Time::getTimestamp()
//------------------------------------------------------------------------------
anna::U32 anna::diameter::codec::basetypes::Time::getTimestamp(Timestamp::_v timestampType) const throw() {
  return ((timestampType == Timestamp::NTP) ? a_ntpTimestamp : (a_ntpTimestamp - TIMESTAMP_OFFSET_NTP1900_OVER_UNIX1970));
}


//------------------------------------------------------------------------------
//--------------------------------------------------------- Time::setTimestamp()
//------------------------------------------------------------------------------
void anna::diameter::codec::basetypes::Time::setTimestamp(const U32& timestamp, Timestamp::_v timestampType) throw() {
  a_ntpTimestamp = (timestampType == Timestamp::NTP) ? timestamp : (timestamp + TIMESTAMP_OFFSET_NTP1900_OVER_UNIX1970);
  updateBasic();
}

